/*global define, CKEDITOR, setTimeout*/
/*jslint regexp: true, unparam: true*/
define(['testsuite/qunit', 'jquery', 'ep/ui/editor', '$ready!'], function (QUnit, $) {
    'use strict';

    QUnit.module("ckeditor/plugins/eptable");

    var delay,
        editor1 = $('#uieditor_ckeditortestinstance_test1'),
        editor2 = $('#uieditor_ckeditortestinstance_test2'),
        editor3 = $('#uieditor_ckeditortestinstance_test3'),
        inputPositions = {
            rows: 0,
            cells: 1,
            borderWidth: 2,
            width: 3,
            height: 4,
            cellpadding: 5,
            summary: 8,
            caption: 9
        };

    function cke_test(setfunction, testfunction, editorId, editorClass) {
        var editorInstance = CKEDITOR.instances[editorId];

        // reset CKE data
        editorInstance.setData('');
        // open eptable dialog
        $('.' + editorClass + ' .cke_button__eptable').trigger('click');

        setTimeout(function () {
            QUnit.equal($('.' + editorClass + ' .cke_dialog').is(':visible'), true, 'table dialog opened');

            setfunction();
            // close dialog
            $('.' + editorClass + ' .cke_dialog_ui_button_ok span').trigger('click');

            setTimeout(function () {
                QUnit.equal($('.' + editorClass + ' .cke_dialog').is(':visible'), false, 'table dialog closed');

                setTimeout(function () {
                    var data = editorInstance.getData();

                    testfunction(data);
                    QUnit.start();
                }, 200);

            }, 200);

            delay = true;
        }, delay ? 100 : 1000);
    }

    // instantiate CKE
    editor1.uiEditor({
        'name' : 'ckeditortestinstance_test1',
        'toolbar': 'Full'
    });
    editor2.uiEditor({
        'name' : 'ckeditortestinstance_test2',
        'toolbar': 'Full'
    });
    editor3.uiEditor({
        'name' : 'ckeditortestinstance_test3',
        'toolbar': 'Full'
    });

    setTimeout(function () {

        QUnit.asyncTest("basic table plugin tests", function () {
            var editorId = 'uieditor_ckeditortestinstance_test1',
                editorClass = 'cke_1';

            cke_test(function () {
                QUnit.equal($('.' + editorClass + ' .cke_dialog').find('a.cke_dialog_tab').length, 2, 'dialog contains 2 tabs');
                QUnit.equal($('.' + editorClass + ' .cke_dialog').find('input:text').length, 13, 'dialog contains 13 text inputs');
                QUnit.equal($('.' + editorClass + ' .cke_dialog').find('select').length, 3, 'dialog contains 3 select elements');
                QUnit.equal($('.' + editorClass + ' .cke_dialog').find('select').length, 3, 'dialog contains 3 select elements');
                QUnit.equal($('.' + editorClass + ' .cke_dialog').find('.epBoderColorCKE').length, 1, 'dialog contains 1 element to chose border color');
                QUnit.equal($('.' + editorClass + ' .cke_dialog').find('.epBackgroundColorCKE').length, 1, 'dialog contains 1 element to chose background color');
            }, function (data) {
                var fragment = $("<div />"),
                    table = $(fragment).html(data).children('table:first');

                QUnit.ok(table.length === 1, 'table was inserted');
                QUnit.ok(table.find('tr').length === 3, 'table has 3 rows');
                QUnit.ok(table.find('tr:first td').length === 2, 'each table row contains 2 cells');
                QUnit.ok(table.css('border-width') === "1px", 'border width is 1 pixel');
                QUnit.ok(table.css('border-style') === 'solid', 'border style is solid');
                QUnit.ok(table.css('border-color') === 'rgb(0, 0, 0)', 'border color is rgb(0,0,0)');
                QUnit.ok(table.attr('cellpadding') === '1', 'cellpadding is set to 1');
            }, editorId, editorClass);
        });

        QUnit.asyncTest("test table plugin with different parameters", function () {
            var editorId = 'uieditor_ckeditortestinstance_test2',
                editorClass = 'cke_2',
                editorInstance = CKEDITOR.instances[editorId];

            // reset CKE data
            editorInstance.setData('');
            // open eptable dialog
            $('.' + editorClass + ' .cke_button__eptable').trigger('click');

            setTimeout(function () {
                QUnit.equal($('.' + editorClass + ' .cke_dialog').is(':visible'), true, 'table dialog opened');

                // open colorpicker dialog for border color
                $('.' + editorClass + ' .cke_dialog').find('.epBoderColorCKE').find('input').trigger('click');

                setTimeout(function () {
                    var colorpickerElements = $('.ep-uiSimpledialog').data('tmplItem').elements;

                    QUnit.equal($('.ep-uiSimpledialog').is(':visible'), true, 'colortool dialog for border color is open');
                    // close colortool dialog again
                    $(colorpickerElements.content[0]).find('.ep-uiColorpicker-applyButton').trigger("click");

                    setTimeout(function () {
                        var inputs = $('.' + editorClass + ' .cke_dialog').find('input:text');

                        // open dialog for background color
                        $('.' + editorClass + ' .cke_dialog').find('.epBackgroundColorCKE').find('input').trigger('click');

                        setTimeout(function () {
                            var colorpickerElements = $('.ep-uiSimpledialog:last').data('tmplItem').elements;

                            QUnit.equal($('.ep-uiSimpledialog').is(':visible'), true, 'colortool dialog for background color is open');

                            // close colortool for background color dialog again
                            $(colorpickerElements.content[0]).find('.ep-uiColorpicker-applyButton').trigger("click");

                            // set border color data
                            $('.' + editorClass + ' .cke_dialog').find('.epBoderColorCKE').find('input').val("#0f0");
                            // set number of rows
                            inputs.eq(inputPositions.rows).val(2);
                            // set number of cells
                            inputs.eq(inputPositions.cells).val(10);
                            // set border-width
                            inputs.eq(inputPositions.borderWidth).val(3);
                            // set width
                            inputs.eq(inputPositions.width).val("100px");
                            // set height
                            inputs.eq(inputPositions.height).val("200px");
                            // set cellpadding
                            inputs.eq(inputPositions.cellpadding).val("0");
                            // set table header
                            inputs.eq(inputPositions.caption).val("here is a table");
                            // set table caption
                            inputs.eq(inputPositions.summary).val("here is a caption");

                            // close cke dialog
                            $('.' + editorClass + ' .cke_dialog_ui_button_ok span').trigger('click');

                            setTimeout(function () {
                                var data = editorInstance.getData(),
                                    fragment = $("<div />"),
                                    table = $(fragment).html(data).children('table:first');

                                QUnit.equal($('.ep-uiColorpicker-dialog').is(':visible'), false, 'colortool dialog for background color is closed');
                                QUnit.equal($('.' + editorClass + ' .cke_dialog').is(':visible'), false, 'table dialog closed');

                                // test border properties
                                QUnit.equal(table.css('border-color'), 'rgb(0, 255, 0)', 'border color is green');
                                QUnit.equal(table.css('border-width'), '3px', 'border width is 3 pixel');

                                // test number of rows and cells
                                QUnit.ok(table.find('tr').length === 2, 'table has 3 rows');
                                QUnit.ok(table.find('tr:first td').length === 10, 'each table row contains 10 cells');

                                // test css/attribute properties
                                QUnit.equal(table.css('height'), '200px', 'height of table is 200 pixel');
                                QUnit.equal(table.css('width'), '100px', 'width of table is 100 pixel');
                                QUnit.equal(table.attr('cellpadding'), 0, 'table has no cellpadding');

                                // test caption
                                QUnit.equal(table.find('caption').length, 1, 'table contains a caption element');
                                // test header
                                QUnit.ok(table.attr('summary') !== undefined, 'table has a summary');
                                QUnit.equal(table.attr('summary'), 'here is a table', 'table\'s summary is\'here is a table\'');

                                QUnit.start();
                            }, 200);

                        }, 200);

                    }, 200);

                }, 200);

            }, 1000);
        });

        QUnit.asyncTest("test table plugin with existing parameters", function () {
            var editorId = 'uieditor_ckeditortestinstance_test3',
                editorClass = 'cke_3',
                editorInstance = CKEDITOR.instances[editorId];

            // reset CKE data
            editorInstance.setData('<table cellpadding="0" class="editor_table_with_border" style="background-color: rgb(239, 239, 239); border: 1px solid rgb(0, 255, 0); width: 100px; height: 200px;"><tbody><tr><td class="editor_td_with_border" style="border: 1px solid rgb(0, 255, 0);">&nbsp;</td><td class="editor_td_with_border" style="border: 1px solid rgb(0, 255, 0);">&nbsp;</td></tr><tr><td class="editor_td_with_border" style="border: 1px solid rgb(0, 255, 0);">&nbsp;</td><td class="editor_td_with_border" style="border: 1px solid rgb(0, 255, 0);">&nbsp;</td></tr></tbody></table>');

            setTimeout(function () {
                // fragment to select the inserted table
                (function (selection) {
                    var element = selection.getStartElement(),
                        range;

                    selection.selectElement(element);
                    range = selection.getRanges()[0];
                    range.startOffset = 0;
                    range.endOffset = 1;
                    selection.selectRanges([range]);
                }(editorInstance.getSelection()));

                // open eptable dialog with existing properties
                editorInstance.execCommand('eptableProperties');

                setTimeout(function () {
                    QUnit.equal($('.' + editorClass + ' .cke_dialog').is(':visible'), true, 'table dialog opened');

                    setTimeout(function () {
                        var inputs = $('.' + editorClass + ' .cke_dialog').find('input:text');

                        // test if table dialog is open
                        QUnit.equal($('.' + editorClass + ' .cke_dialog').is(':visible'), true, 'table dialog opened');

                        // test to check if already existing properties are inserted to the dialog elements
                        QUnit.equal($('.' + editorClass + ' .cke_dialog').find('.epBoderColorCKE').find('input').val(), 'rgb(0, 255, 0)', 'border color is green');
                        QUnit.equal($('.' + editorClass + ' .cke_dialog').find('.epBackgroundColorCKE').find('input').val(), 'rgb(239, 239, 239)', 'background color is rgb(239, 239, 239)');
                        QUnit.equal(inputs.eq(inputPositions.rows).val(), 2, 'table contains 2 rows');
                        QUnit.equal(inputs.eq(inputPositions.cells).val(), 2, 'each row contains 2 cells');
                        QUnit.equal(inputs.eq(inputPositions.borderWidth).val(), 1, 'border width of the table is set to 1');
                        QUnit.equal(inputs.eq(inputPositions.width).val(), '100px', 'width of table is set to 100 pixel');
                        QUnit.equal(inputs.eq(inputPositions.height).val(), '200px', 'height of table is 200 pixel');
                        QUnit.equal(inputs.eq(inputPositions.cellpadding).val(), '0', 'cellpadding of table is 0');


                        // close cke dialog
                        $('.' + editorClass + ' .cke_dialog_ui_button_ok span').trigger('click');

                        setTimeout(function () {
                            QUnit.start();
                        }, 200);

                    }, 200);

                }, 200);

            }, 1000);
        });

    }, 2000);
});